@@ -6,7 +6,6 @@ use super::process::Process;
 use super::{get_windir, predicate, str_from_wide_str};
 use crate::error::Error;
 use crate::Result;
-use log::{debug, error, info, trace, warn};
 use ntapi::ntpsapi::PROCESS_BASIC_INFORMATION;
 use ntapi::ntwow64::LDR_DATA_TABLE_ENTRY32;
 use once_cell::sync::OnceCell;
@@ -14,14 +13,13 @@ use pelite::Wrap;
 use std::ffi::OsStr;
 use std::ops::Shl;
 use std::os::windows::ffi::OsStrExt;
-use std::ptr::null_mut;
 pub use types::LDR_DATA_TABLE_ENTRY64;
-use winapi::shared::basetsd::{DWORD64, PDWORD64, PULONG64, ULONG64};
 use winapi::shared::minwindef::{HMODULE, PULONG, ULONG};
-use winapi::shared::ntdef::{NTSTATUS, PVOID, PVOID64};
+use winapi::shared::ntdef::{NTSTATUS, PVOID};
 use winapi::um::libloaderapi::{FreeLibrary, GetProcAddress, LoadLibraryW};
-use winapi::um::processthreadsapi::GetCurrentProcess;
-use winapi::um::winnt::{HANDLE, PROCESS_QUERY_INFORMATION, PROCESS_QUERY_LIMITED_INFORMATION};
+use winapi::um::winnt::{
+    HANDLE, PROCESS_QUERY_INFORMATION, PROCESS_QUERY_LIMITED_INFORMATION, PROCESS_VM_READ,
+};
 
 ///This class represents the NTDLL module/dll from windows.
 ///This class is focused on having the least possible NTDLL function calls,
@@ -75,35 +73,6 @@ impl NTDLL {
         }
     }
 
-    ///Runs get_module_in_proc
-    ///For safety go to [get_module_in_proc]
-    ///
-    ///proc: A Process (handle)
-    ///predicate: when true is seen, selector is invoked
-    ///selector: The selector returns the desired information
-    pub(crate) unsafe fn run_get_module_in_proc<S, P, E, R>(
-        &self,
-        proc: &Process,
-        selector: S,
-        predicate: P,
-    ) -> Result<R>
-    where
-        S: Fn(
-            Wrap<ntapi::ntwow64::LDR_DATA_TABLE_ENTRY32, types::LDR_DATA_TABLE_ENTRY64>,
-            &Vec<u16>,
-        ) -> R,
-        P: Fn(&Vec<u16>) -> bool,
-        E: Fn(Error) -> Result<R>,
-    {
-        self.get_module_in_proc(proc, |w, v| {
-            if predicate(&v) {
-                Some(selector(w, &v))
-            } else {
-                None
-            }
-        })
-    }
-
     ///Gets a module, by reading the Process Environment Block (PEB), of the process, using ntdll functions.
     ///Because this function uses ntdll functions, it should work the same, if running as x86, or x64.
     ///
@@ -122,16 +91,12 @@ impl NTDLL {
             Vec<u16>,
         ) -> Option<R>,
     {
-        if !proc.has_real_handle() {
-            return Err(crate::error::Error::Io(std::io::Error::from(
-                std::io::ErrorKind::InvalidInput,
-            )));
-        }
+        proc.err_pseudo_handle()?;
         let pid_under_wow = proc.is_under_wow()?;
-        info!("pid is under wow:{}", pid_under_wow);
+        crate::info!("pid is under wow:{}", pid_under_wow);
         let peb_addr: u64=
         //This gets the PEB address, from the PBI
-        if proc.is_under_wow()?{
+        if pid_under_wow {
             let pbi: PROCESS_BASIC_INFORMATION=
                 self.query_process_information(proc, ntapi::ntpsapi::ProcessBasicInformation)?;
             pbi.PebBaseAddress as u64
@@ -140,7 +105,7 @@ impl NTDLL {
                 self.query_process_information(proc, ntapi::ntpsapi::ProcessBasicInformation)?;
             pbi.PebBaseAddress as u64
         };
-        debug!("Peb addr is {:x?}", peb_addr);
+        crate::debug!("Peb addr is {:x?}", peb_addr);
         let ldr_addr;
         //This reads the PEB, and gets the LDR address
         {
@@ -151,9 +116,25 @@ impl NTDLL {
                 peb.Ldr as u64
             } else {
                 let peb = *self.read_virtual_mem::<PEB64>(proc, peb_addr)?;
+                //Since this is the structure of PEB under 64-bits, this should always hold true
+                //A good test, if peb structure is valid.
+                //https://docs.microsoft.com/en-us/windows/win32/api/Winternl/ns-winternl-peb
+                if *(&peb as *const PEB64 as *const u8 as *const u64).add(3) != peb.Ldr {
+                    //If we get here, it means, that the PEB struct we use is invalid.
+                    #[allow(deprecated)]
+                    //There really is not any other way to describe this error.
+                    return Err(crate::error::Error::Unsuccessful(Some(
+                        "The PEB structure in inject-lib is invalid.".to_string(),
+                    )));
+                }
                 peb.Ldr as u64
             };
-            debug!("Ldr Address is {:x}.", ldr_addr);
+            if ldr_addr == 0 {
+                return Err(crate::error::Error::Unsuccessful(Some(
+                    "LDR is not initialised?".to_string(),
+                )));
+            }
+            crate::debug!("Ldr Address is {:x}.", ldr_addr);
         }
 
         let mut modlist_addr;
@@ -170,7 +151,7 @@ impl NTDLL {
                 let ldr = *self.read_virtual_mem::<PEB_LDR_DATA64>(proc, ldr_addr)?;
                 ldr.InLoadOrderModuleList.Flink as u64
             };
-            debug!("Ldr InLoadOrderModuleList Address is {:x}", modlist_addr);
+            crate::debug!("Ldr InLoadOrderModuleList Address is {:x}", modlist_addr);
         }
         let first_modlist_addr = modlist_addr;
         //This Loops through the Module list, until we have found our module, or we arrive, at the address, we started from.
@@ -182,11 +163,11 @@ impl NTDLL {
             let ldr_entry_data: Wrap<LDR_DATA_TABLE_ENTRY32, LDR_DATA_TABLE_ENTRY64>;
             if pid_under_wow {
                 let entry = *self.read_virtual_mem(proc, modlist_addr as u64)?;
-                debug!("Read the LDR_DATA_Table {:#x}", modlist_addr);
+                crate::debug!("Read the LDR_DATA_Table {:#x}", modlist_addr);
                 ldr_entry_data = Wrap::T32(entry);
             } else {
                 let entry = *self.read_virtual_mem(proc, modlist_addr as u64)?;
-                debug!("Read the LDR_DATA_Table {:#x}", modlist_addr);
+                crate::debug!("Read the LDR_DATA_Table {:#x}", modlist_addr);
                 ldr_entry_data = Wrap::T64(entry);
             }
             {
@@ -210,7 +191,7 @@ impl NTDLL {
                 };
                 if modlist_addr == first_modlist_addr {
                     const RECURSION:&str = "We looped through the whole InLoadOrderModuleList, but still have no match. Aborting, because this would end in an endless loop.";
-                    warn!("{}", RECURSION);
+                    crate::warn!("{}", RECURSION);
                     return Err(Error::Unsuccessful(Some(RECURSION.to_string())));
                 }
 
@@ -234,10 +215,10 @@ impl NTDLL {
                 };
                 match str_from_wide_str(dll_path.as_slice()) {
                     Ok(v) => {
-                        debug!("dll_name is {},{:x}", v, addr);
+                        crate::debug!("dll_name is {},{:x}", v, addr);
                     }
                     Err(e) => {
-                        debug!("dll_name could not be printed. os_string is {:?}", e);
+                        crate::debug!("dll_name could not be printed. os_string is {:?}", e);
                     }
                 }
                 if let Some(val) = predicate(ldr_entry_data, dll_path) {
@@ -248,7 +229,11 @@ impl NTDLL {
     }
 
     ///See [read_virtual_mem_fn].
-    unsafe fn read_virtual_mem<T>(&self, proc: &Process, addr: u64) -> Result<*mut T> {
+    pub(in super::super) unsafe fn read_virtual_mem<T>(
+        &self,
+        proc: &Process,
+        addr: u64,
+    ) -> Result<*mut T> {
         let v = self.read_virtual_mem_fn(proc, addr, core::mem::size_of::<T>() as u32)?;
         Ok(v.leak().as_ptr() as *mut T)
     }
@@ -267,10 +252,16 @@ impl NTDLL {
     ///
     /// T needs to be non zero sized.
     ///
-    unsafe fn read_virtual_mem_fn(&self, proc: &Process, addr: u64, size: u32) -> Result<Vec<u8>> {
-        if !proc.has_real_handle() {
+    pub(in super::super) unsafe fn read_virtual_mem_fn(
+        &self,
+        proc: &Process,
+        addr: u64,
+        size: u32,
+    ) -> Result<Vec<u8>> {
+        proc.err_pseudo_handle()?;
+        if !proc.has_perm(PROCESS_VM_READ) {
             return Err(crate::error::Error::Io(std::io::Error::from(
-                std::io::ErrorKind::InvalidInput,
+                std::io::ErrorKind::PermissionDenied,
             )));
         }
         static FNS: OnceCell<FnNtdllWOW> = OnceCell::new();
@@ -278,44 +269,49 @@ impl NTDLL {
             FnNtdllWOW::new(b"NtReadVirtualMemory\0", b"NtWow64ReadVirtualMemory64\0")
         })?;
         let mut buf: Vec<u8> = Vec::with_capacity(size as usize);
-        trace!("reading at address {:x?} {} bytes", addr, size);
-        println!("reading at address {:x?} {} bytes", addr, size);
-        let mut i: u32 = 0;
-        let status = if proc.is_under_wow()? {
-            let cfn: types::FnNtReadVirtualMemory = core::mem::transmute(*fns.get_fn()?.take());
-            cfn(
-                proc.get_proc(),
-                addr as PVOID,
-                buf.as_mut_ptr() as PVOID,
-                size as u32,
-                &mut i as *mut u32,
-            )
+        crate::trace!("reading at address {:x?} {} bytes", addr, size);
+        let mut i: u64 = 0;
+        let func = if proc.is_under_wow()? {
+            fns.get_fn()?
         } else {
-            let cfn: types::FnNtWOW64ReadVirtualMemory64 =
-                core::mem::transmute(*fns.get_fn()?.take());
-            cfn(
-                proc.get_proc(),
-                addr,
-                buf.as_mut_ptr() as PVOID64,
-                size as u32,
-                &mut i as *mut u32 as u64,
-            )
+            fns.get_wow64()?
         };
+        let status = match func {
+            NtdllFn::Normal(v) => {
+                let cfn: types::FnNtReadVirtualMemory = core::mem::transmute(*v);
+                cfn(
+                    proc.get_proc(),
+                    addr as PVOID,
+                    buf.as_mut_ptr() as PVOID,
+                    size as u32,
+                    &mut i as *mut u64 as *mut u32,
+                )
+            }
+            NtdllFn::WOW(v) => {
+                let cfn: types::FnNtWOW64ReadVirtualMemory64 = core::mem::transmute(*v);
+                cfn(
+                    proc.get_proc(),
+                    addr,
+                    buf.as_mut_ptr() as PVOID,
+                    size as u64,
+                    &mut i as *mut u64,
+                )
+            }
+        };
+        let status = crate::error::Ntdll::new(status);
+        crate::trace!("rvm {},{}/{}", status, i, size);
+        if status.is_warning() || status.is_error() {
+            return Err(status.into());
+        }
 
-        trace!("rvm {},{}/{}", status, i, size);
-        println!("rvm {},{}/{}", status, i, size);
         //We read i bytes. So we let the Vec know, so it can calculate size and deallocate accordingly, if it wants.
         //Also: This will enable debugger inspection, of the buf, since the debugger will now know, that the vec is initialised.
         assert!(
-            i <= size as u32,
+            i <= size as u64,
             "{:#x} bytes were read into a {:#x} byte buffer.",
             i,
             size
         );
-        // if i==0{
-        //     println!("zero bytes read?");
-        //     i=size as u64;
-        // }
         buf.set_len(i as usize);
         buf.shrink_to_fit();
         Ok(buf)
@@ -344,13 +340,9 @@ impl NTDLL {
     where
         T: Copy,
     {
-        if !proc.has_real_handle() {
-            return Err(crate::error::Error::Io(std::io::Error::from(
-                std::io::ErrorKind::InvalidInput,
-            )));
-        }
+        proc.err_pseudo_handle()?;
         if !proc.has_perm(PROCESS_QUERY_INFORMATION)
-            || !proc.has_perm(PROCESS_QUERY_LIMITED_INFORMATION)
+            && !proc.has_perm(PROCESS_QUERY_LIMITED_INFORMATION)
         {
             return Err(crate::error::Error::Io(std::io::Error::from(
                 std::io::ErrorKind::PermissionDenied,
@@ -384,7 +376,7 @@ impl NTDLL {
         let size_peb: usize = std::mem::size_of::<T>();
         let mut buf: Vec<u8> = Vec::with_capacity(size_peb);
         //Call function
-        println!("Running NtQueryInformationProcess with fnptr:{:x?} proc:{:x?},pic:{:x}. Size is {}, buf is {:x?}",cfn as usize,proc.get_proc(),pic,size_peb, buf);
+        crate::trace!("Running NtQueryInformationProcess with fnptr:{:x?} proc:{:x?},pic:{:x}. Size is {}, buf is {:x?}",cfn as usize,proc.get_proc(),pic,size_peb, buf);
         let status = crate::error::Ntdll::new(cfn(
             proc.get_proc(),
             pic,
@@ -392,14 +384,9 @@ impl NTDLL {
             size_peb as u32,
             i_ptr,
         ));
-        println!(
-            "qip {:x},0x{:x}|0x{:x}/0x{:x} buf is {:?}",
-            status.get_status(),
-            i,
-            i,
-            size_peb,
-            buf
-        );
+        if status.is_error() || status.is_warning() {
+            return Err(status.into());
+        }
         assert!(i<=size_peb as u32,"Detected buffer overflow. Stopping here, due to unknown or possibly malicious side-effects.");
         if i == 0 && size_peb != 0 {
             return Err(crate::error::Error::Unsuccessful(Some(
@@ -409,6 +396,13 @@ impl NTDLL {
         //Truncate vec, to only use initialized memory.
         buf.set_len(i as usize);
         buf.shrink_to_fit();
+        crate::trace!(
+            "qip {:x},0x{:x}/0x{:x} buf is {:x?}",
+            status.get_status(),
+            i,
+            size_peb,
+            buf
+        );
         //This should be safe, since the vec has as many bytes, as T
         let pbi_ptr: *mut T = std::mem::transmute(buf.leak().as_mut_ptr());
         // trace!("exitstatus:{:x},pebaddress:{:x},baseprio:{:x},upid:{:x},irupid:{:x}",pbi.ExitStatus,pbi.PebBaseAddress,pbi.BasePriority,pbi.UniqueProcessId,pbi.InheritedFromUniqueProcessId);
@@ -466,6 +460,7 @@ impl<'a, 'b, 'c> FnNtdllWOW<'a, 'b, 'c> {
         }
         #[cfg(target_pointer_width = "32")]
         {
+            crate::trace!("wow64 fn");
             self.wowfn
                 .get_or_try_init(|| {
                     Ok(std::mem::transmute(check_ptr!(GetProcAddress(
@@ -478,6 +473,7 @@ impl<'a, 'b, 'c> FnNtdllWOW<'a, 'b, 'c> {
     }
     ///returns a function which is the NtReadVirtualMemory function inside NTDLL
     pub(self) unsafe fn get_fn(&self) -> Result<NtdllFn<&usize>> {
+        crate::trace!("regular fn");
         self.namefn
             .get_or_try_init(|| {
                 Ok(std::mem::transmute(check_ptr!(GetProcAddress(
@@ -494,13 +490,13 @@ impl Drop for NTDLL {
     ///Also invalidates handle.
     fn drop(&mut self) {
         if unsafe { FreeLibrary(self.handle as HMODULE) } == 0 {
-            warn!("Error whilst unloading NTDLL.dll. This is not actually that bad, since it is present in every Process anyways.");
+            crate::warn!("Error whilst unloading NTDLL.dll. This is not actually that bad, since it is present in every Process anyways.");
             err("FreeLibrary");
         };
     }
 }
 #[cfg(test)]
-mod test {
+pub mod test {
     use super::NTDLL;
     use crate::Result;
     use ntapi::ntpsapi::PROCESS_BASIC_INFORMATION;
@@ -515,10 +511,8 @@ mod test {
 
     #[test]
     fn get_module_in_proc() -> Result<()> {
-        let proc = super::super::process::Process::new(std::process::id(), PROCESS_ALL_ACCESS)?;
-        // let proc = super::super::process::Process::self_proc();
         let ntdll = NTDLL::new()?;
-        let (b, m) = unsafe {
+        let test = |proc: &super::super::process::Process| unsafe {
             ntdll.get_module_in_proc(
                 &proc,
                 super::predicate(
@@ -529,10 +523,22 @@ mod test {
                         pelite::Wrap::T32(x) => x.DllBase as u64,
                         pelite::Wrap::T64(x) => x.DllBase as u64,
                     },
-                    |x| super::super::cmp("KERNEL32.DLL")(&x),
+                    |x| super::super::cmp("ntdll.dll")(&x),
                 ),
             )
-        }?;
+        };
+        //test self real-handle
+        {
+            let proc = super::super::process::Process::new(std::process::id(), PROCESS_ALL_ACCESS)?;
+            test(&proc)?;
+        }
+        //test foreign
+        {
+            let (mut c, proc) = super::super::test::create_cmd();
+            let r = test(&proc);
+            c.kill().unwrap();
+            assert!(r.is_ok(), "other: {}", r.unwrap_err());
+        }
         Ok(())
     }
     #[test]
@@ -551,36 +557,74 @@ mod test {
 
     #[test]
     fn read_memory() -> Result<()> {
-        let buf: Vec<u8> = (0..255).collect();
-        let s = buf.as_slice();
-        let ntdll = NTDLL::new()?;
-        let proc = super::super::process::Process::new(std::process::id(), PROCESS_ALL_ACCESS)?;
-        let self_proc = super::super::process::Process::self_proc();
-        let mut mem = super::super::mem::MemPage::new(&proc, buf.len(), false)?;
-        mem.write(s)?;
-        let re = unsafe {
-            ntdll.read_virtual_mem_fn(self_proc, mem.get_address() as u64, buf.len() as u32)
-        };
-        assert!(
-            re.is_err(),
-            "read_virtual_mem_fn does apparently not work without a real handle"
-        );
+        //test read_mem on self
+        {
+            let buf: Vec<u8> = (0..255).collect();
+            let s = buf.as_slice();
+            let ntdll = NTDLL::new()?;
+            //Test self proc
+            {
+                let self_proc = super::super::process::Process::self_proc();
+                let re = unsafe {
+                    ntdll.read_virtual_mem_fn(self_proc, s.as_ptr() as u64, buf.len() as u32)
+                };
+                assert!(
+                    re.is_err(),
+                    "read_virtual_mem_fn does apparently not work without a real handle"
+                );
+            }
+            //test proc real handle self
+            {
+                let proc =
+                    super::super::process::Process::new(std::process::id(), PROCESS_ALL_ACCESS)?;
+                let r = unsafe {
+                    ntdll.read_virtual_mem_fn(&proc, s.as_ptr() as u64, buf.len() as u32)
+                }?;
+                assert_eq!(r, buf);
+            }
+            //test foreign process
+            {
+                let (mut c, proc) = super::super::test::create_cmd();
+                let mut mem = super::super::mem::MemPage::new(&proc, buf.len(), false)?;
+                let s = mem.write(buf.as_slice())?;
+                let r = unsafe {
+                    ntdll.read_virtual_mem_fn(&proc, mem.get_address() as u64, s as u32)
+                }?;
+                c.kill().unwrap();
+                assert_eq!(r, buf, "Foreign read failed");
+            }
+        }
 
-        println!("mem addr at {:#x}", mem.get_address() as usize);
-        let r = unsafe { ntdll.read_virtual_mem_fn(&proc, s.as_ptr() as u64, buf.len() as u32) }?;
-        assert_eq!(r, buf);
         Ok(())
     }
 
     #[test]
     fn query_process_information_self() -> Result<()> {
         let ntdll = super::NTDLL::new()?;
+        //test real handle self
         {
             let proc = super::super::process::Process::new(std::process::id(), PROCESS_ALL_ACCESS)?;
-            let r: PROCESS_BASIC_INFORMATION = unsafe {
-                ntdll.query_process_information(&proc, ntapi::ntpsapi::ProcessBasicInformation)?
+            let r = unsafe {
+                ntdll.query_process_information::<PROCESS_BASIC_INFORMATION>(
+                    &proc,
+                    ntapi::ntpsapi::ProcessBasicInformation,
+                )
             };
+            assert!(r.is_ok(), "self query process information failed");
         }
+        //test foreign process
+        {
+            let (mut c, proc) = super::super::test::create_cmd();
+            let r = unsafe {
+                ntdll.query_process_information::<PROCESS_BASIC_INFORMATION>(
+                    &proc,
+                    ntapi::ntpsapi::ProcessBasicInformation,
+                )
+            };
+            assert!(r.is_ok(), "foreign query process information failed");
+            c.kill().unwrap();
+        }
+        //test pseudo handle self
         {
             let proc = super::super::process::Process::self_proc();
             let r: Result<PROCESS_BASIC_INFORMATION> = unsafe {
